.macpack generic
.include "constants.ca65.inc"	
;;;----------------------------------------------------------------
;;; constants
;;;----------------------------------------------------------------
PPUADDR = $2006
PPUDATA = $2007	
;;;----------------------------------------------------------------
;;; variables
;;;----------------------------------------------------------------

;;;----------------------------------------------------------------
;;; iNES header
;;;----------------------------------------------------------------

.segment "HEADER"
	.byte "NES", $1a
	.byte 1,0,$70,8
	.byte 0,0,0,4
	.byte 0,0,0,0
.segment "VECTORS"
	.word NMI, Reset, IRQ

;;;----------------------------------------------------------------
;;; program bank(s)
;;;----------------------------------------------------------------

.zeropage
indir:	.word 0
Occured:	.byte 0
outside:	.byte 0
	
.CODE
.proc uploadchrfor218
	lda #0
	sta PPUADDR
	lda #0
	sta PPUADDR

	lda #8
	sta outside
@repeat: 
	lda #>chr
	sta indir+1
	lda #<chr
	sta indir
	ldx #64
	ldy #0
@first:
	lda (indir),y
	sta PPUDATA
	iny
	cpy #8
	bne @first
	ldy #0
@second:
	lda (indir),y
	sta PPUDATA
	iny
	cpy #8
	bne @second

	lda indir
	clc
	adc #8
	sta indir
	lda indir+1
	adc #0
	sta indir+1
	ldy #0
	dex
	bne @first

	dec outside
	bnz @repeat
	rts
.endproc

.proc chr
	.incbin "alnum.1bp"
.endproc

.CODE
.setcpu "6502x"	
WaitVblank:	
	LDA $2002
	BPL WaitVblank
	rts

Reset:
	SEI
	CLD
	LDX #$FF
	TXS
	INX
	STX $2000
	STX $2001
	
	;;Init RAM
	lda #$00
	ldx #$00
:
	sta $00,x
	sta $100,x
	lda #$f0
	sta $200,x
	lda #0
	sta $300,x
	sta $400,x
	sta $500,x
	sta $600,x
	sta $700,x
	dex
	bne :-

	bit $2002
	jsr WaitVblank
	jsr WaitVblank

	jsr uploadchrfor218
	
	;; m218: only init one nametable
	LDA #$20
	STA $2006
	lda #0
	STA $2006

	lda #' '
	LDY #4
	LDX #$00
:
	STA $2007
	DEX
	BNE :-
	
	DEY
	BNE :-
	
	
	;;Init sprites
	LDA #$02
	STA $4014
	
	;;Init palette
	LDA #$3F
	STA $2006
	LDA #$00
	STA $2006  

	LDA #$0F
	STA $2007  
	LDA #$00
	STA $2007
	LDA #$10
	STA $2007  
	LDA #$20
	STA $2007  
	
	JSR WaitVblank
	
	;;Turn on 4-step mode and enable CPU frame IRQ assertion
	LDA #$00
	STA $4017
	
	;;Wait a few frames
	JSR WaitVblank  
	JSR WaitVblank
	
	;;Turn on 5-step mode
	LDA #$80
	STA $4017
	
	;;Enable IRQs
	CLI         ; an IRQ should have happened now, right?
	
	;;Wait a few frames
	JSR WaitVblank  
	JSR WaitVblank

	;;Setup NT write  
	LDA #$24
	STA $2006
	LDA #$A5
	STA $2006
	
	;;If turning on 5-step mode acknowledges the IRQ signal, no IRQ should ever occur.  
	LDA Occured
	CMP #0
	BEQ Success

Failure:
	;;Write "1 FAIL" if IRQ count > 0
	LDA #'1'
	STA $2007
	
	LDA #' '
	STA $2007
	LDA #'F'
	STA $2007
	LDA #'A'
	STA $2007
	LDA #'I'
	STA $2007
	LDA #'L'
	STA $2007  
	JMP End
	
Success:
	;;Write "0 PASS" if IRQ count == 0
	LDA #'0'
	STA $2007
	
	LDA #' '
	STA $2007
	LDA #'P'
	STA $2007
	LDA #'A'
	STA $2007
	LDA #'S'
	STA $2007
	LDA #'S'
	STA $2007

End:
	LDA #$24
	STA $2006
	LDA #$00
	STA $2006
	
	;;Turn on PPU
	LDA #$1E
	STA $2001

:
	JMP :-


NMI:
	RTI

IRQ:
	INC Occured
	;;Acknowledge IRQ
	BIT $4015
	RTI
